-- 	Copyright  2009.	GSI Technology
-- 						Jeff Daugherty
-- 						apps@gsitechnology.com
--  1.0  09/11/06  Jeff Daugherty  1) Created
--  1.1  09/23/09  Jeff Daugherty  1) Model no longers simulates full memory
--                                    array.  Passable constant A_BITS defines
--                                    memroy array as 2^A_BITS
--                                 2) Device speed option is passable as
--                                    constant SPEED.  This will setup speed
--                                    dependent variables without the need to
--                                    modify model.
--  1.2  12/15/10  Jeff Daugherty  1) A_bits controls array size simulated:
--                                    If A_bits > 1 then array size is 2^A_bits-1
--                                    If A_bits <= 1 then array size is actual device density
LIBRARY ieee;
USE ieee.std_logic_1164.all;
  use ieee.std_logic_unsigned.all, ieee.std_logic_arith.all;
ENTITY G81302T38E IS	
  GENERIC (
    CONSTANT A_bits      : integer := 1;--  See Rev 1.2 comment
    constant SPEED       : integer := 450);  -- 450, 400, 375, 333, 300
  PORT (
    SIGNAL A8130: IN std_logic_vector(20 DOWNTO 0);-- address
    SIGNAL DQ   : inout std_logic_vector(35 DOWNTO 0);-- Input  data
    SIGNAL nBW  : IN std_logic_vector(3 downto 0);-- bank 1 write enable
    SIGNAL K    : IN std_logic;-- input clock
    SIGNAL nK   : IN std_logic;-- input clock bar
    SIGNAL nLD  : IN std_logic;-- read enable
    SIGNAL RnW  : IN std_logic;-- write enable
    SIGNAL nDoff: IN std_logic;-- write enable
    SIGNAL CQ   : INOUT std_logic;-- write enable
    SIGNAL nCQ  : INOUT std_logic;-- write enable
    SIGNAL QVLD : INOUT std_logic;-- write enable
    SIGNAL TMS  : IN std_logic;-- Scan Test Mode Select
    SIGNAL TDI  : IN std_logic;-- Scan Test Data In
    SIGNAL TDO  : OUT std_logic;-- Scan Test Data Out
    SIGNAL TCK  : IN std_logic);-- Scan Test Clock
END G81302T38E;

ARCHITECTURE CIO_B2_PLUS_144MEG OF G81302T38E IS
  component PLUS_CIO_B2_CORE
    generic (
     CONSTANT A_size      : integer;
     constant mem_size    : integer;
     CONSTANT A_bits      : integer;
     CONSTANT DQ_width    : integer;
     CONSTANT RL          : integer;
     constant MEM_STYLE   : string(7 downto 1) := "NORMAL ");  -- REDUCED or NORMAL
    port (
      signal       A        : in    std_logic_vector(A_size - 1 downto 0);  -- address
      signal       DQ       : inout std_logic_vector(DQ_width - 1 downto 0); --input data
      signal       cBW      : in    std_logic_vector (3 downto 0);  -- bank 1 write enable
      signal       K        : in    std_logic;  -- input clock
      signal       nK       : in    std_logic;  -- input clock bar
      signal       nLD      : in    std_logic;  -- read enable
      signal       RnW      : in    std_logic;  -- write enable
      signal       nDoff    : in    std_logic;  -- write enable
      signal       CQ       : out   std_logic;  -- write enable
      signal       nCQ      : out   std_logic;  -- write enable
      signal       QVLD     : out   std_logic;  -- write enable
      signal       TCK      : in    std_logic;  -- write enable
      signal       HighZ    :       std_logic_vector(DQ_width - 1 downto 0) ;
      signal       tCHQV    :       time;
      signal       tCHQX    :       time;
      signal       tCQHQV   :       time;
      signal       tCQHQX   :       time;
      signal       tCHQV_off:       time;
      signal       tCHQX_off:       time);
   end component;

  function pick (A_bits, A_size : integer) return integer is
  begin
    if A_bits > 1 then
      return A_bits;
    else
      return A_size + 1;
    end if;
  end pick;

  function pick2 (A_bits: integer; norm, redu:string(7 downto 1)) return string is
  begin
    if A_bits > 1 then
      return redu;
    else
      return norm;
    end if;
  end pick2;

  constant DQ_width  : integer := 36;
  constant A_size    : integer := 21;
  constant RL        : integer := 25;          -- Read Latency
  constant mem_size  : integer := pick(A_bits, A_size);
  constant mem_style : string(7 downto 1) := pick2(A_bits, "NORMAL ", "REDUCED");

  SIGNAL HighZ    : std_logic_vector(DQ_width - 1 downto 0) := (others => 'Z');
  signal tCHQV, tCHQX, tCQHQV, tCQHQX, tCHQV_off, tCHQX_off : time := 0.0 ns;

begin
  tCHQV     <=  0.45 ns;
  tCHQX     <= -0.45 ns;
  tCHQV_off <=  0.00 ns;
  tCHQX_off <=  0.00 ns;
  tCQHQV <=   0.2 ns;
  tCQHQX <=  -0.2 ns;
  assert (SPEED=450 or SPEED=400 or SPEED=375 or SPEED=333 or SPEED=300) report "Invalid SPEED option, allowed SPEEDs are 450, 400, 375, 333, 300" severity failure;
  QUAD_CORE_CALL : PLUS_CIO_B2_CORE
    generic map (
      mem_size  => mem_size,
      mem_style => mem_style,
      A_size    => A_size,
      A_bits    => A_bits,
      RL        => RL,
      DQ_width  => DQ_width)
    port map (A8130, DQ, nBW, K, NK, NLD, RNW, nDoff, CQ, nCQ, QVLD, TCK, HighZ, tCHQV, tCHQX, tCQHQV, tCQHQX, tCHQV_off, tCHQX_off);
END CIO_B2_PLUS_144MEG;
